% file that downloads the data from the data directory and transforms the
% data into a suitable form for the codes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% data download
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cd data
load growth.txt             % growth
load open.txt               % openess
load dist.txt               % geographic distance
load polity.txt             % polity IV data
load pcgdp1.txt; pcgdp=pcgdp1;              % gdp per capita
load CivilWars.txt
load SovietControl.txt
load CivilWarsNeigh.txt
load PolityNeigh.txt
load deficit.txt
load inflation.txt
load education.txt

% loading the VCM of growth shocks (estimated with the hierarchical model)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load SIGMA_May29_prior0
VCM=SIGMA_May29_prior0;
clear SIGMA_May29_prior0;
cd ..

% for gdp per capita we compute moving average
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[T N]=size(pcgdp);
npma=4;             % number of periods for moving average - 1
PCGDP=zeros(T,N);
pcgdp=[-999*ones(npma,N);pcgdp];
ind_pcgdp=pcgdp~=-999;
for Tau=npma+1:T+npma 
    PCGDP(Tau-npma,:)=sum(pcgdp(Tau-npma:Tau,:).*ind_pcgdp(Tau-npma:Tau,:))./sum(ind_pcgdp(Tau-npma:Tau,:));
end
nus=125;
PCGDP=log(PCGDP)-repmat(log(PCGDP(:,nus)),1,N);
PCGDP(find(isnan(PCGDP)))=-999;

% for inflation we compute moving average
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[T N]=size(inflation);
npma=4;             % number of periods for moving average - 1
INF=zeros(T,N);
inflation=[-999*ones(npma,N);inflation];
ind_inflation=inflation~=-999;
for Tau=npma+1:T+npma 
    INF(Tau-npma,:)=sum(inflation(Tau-npma:Tau,:).*ind_inflation(Tau-npma:Tau,:))./sum(ind_inflation(Tau-npma:Tau,:));
end
INF=INF/100;
INF(find(isnan(INF)))=-999;

% dividing deficit by 100
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
DEF=deficit/100;