function [g badg]=gradient(estpr,Y,X,T,N,ind,priorvec,distance,k,kpol,diagsqrtVCM,XX,XY,regPOL,FT);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%profile on

% parameters bounds and econometrician's priors
b0mean=priorvec(1:2);
b0std=priorvec(3);
scaleS=priorvec(4);
degS=priorvec(5);
minS=priorvec(6);
degk=priorvec(7);
scalek=priorvec(8);
minsigmak=priorvec(9);
maxcorr=priorvec(10);
FEmean=priorvec(11);
FEstd=priorvec(12);


% parameter definitions
b0=estpr(1:2*N);
S=exp(estpr(2*N+1:3*N))+minS;
sigmak=exp(estpr(3*N+1:4*N))+minsigmak;
bdist=exp(estpr(4*N+1:4*N+k));
if kpol>=1
    bpol=estpr(4*N+k+1:4*N+k+kpol);
    GRADBpol=zeros(kpol,1);
else
    GRADBpol=[];
end
if FT==0;
    fixedeffect=zeros(N,1);
    timeeffect=zeros(T,1);
elseif FT==1
    fixedeffect=estpr(4*N+k+kpol+1:5*N+k+kpol);
    timeeffect=zeros(T,1);
    GRADFE=zeros(N,1);
elseif FT==2
    fixedeffect=estpr(4*N+k+kpol+1:5*N+k+kpol);
    timeeffect=estpr(5*N+k+kpol+1:5*N+k+kpol+T);
    GRADFE=zeros(N,1);
    GRADTE=zeros(T,1);
end


% constructing the prior VCM of the betas
BDIST=repmat(permute(bdist,[3 2 1]),[N,N,1]);
v=maxcorr*exp(-sum(distance(:,:,1:k).*BDIST,3)); v=v-diag(diag(v))+eye(N);
if isinf(bdist)==1;
    v=eye(N);
end
Pold=(kron(eye(2),diag(1./S)*diag(1./diagsqrtVCM)/v*diag(1./diagsqrtVCM)*diag(1./S)));
invV=Pold;
VV=(diag(diagsqrtVCM)*v*diag(diagsqrtVCM));
bold=b0;


% gradient vectors
GRADb0=zeros(2*N,1);
GRADS=zeros(N,1);
GRADsigmak=zeros(N,1);
GRADBdist=zeros(k,1);


% recursion
loglh=0;
for t=1:T

    % updating beliefs
    Pnew=Pold+XX(:,(t-1)*2*N+1:t*2*N);
    boldG=bold(1:N);
    boldM=bold(N+1:end);
    bnew=Pnew\(Pold*bold+XY(:,t));
   % bnew=[Pnew(1:N,1:N)\(Pold(1:N,1:N)*boldG+XY(1:N,t));Pnew(N+1:end,N+1:end)\(Pold(N+1:end,N+1:end)*boldM+XY(N+1:end,t))];

    
    % lh computation
    if kpol>=1
        regPOLt=regPOL([t:T:t+(kpol-1)*T],ind(t,:));
        PSI=normcdf(boldM(ind(t,:))-boldG(ind(t,:))-timeeffect(t)-fixedeffect(ind(t,:))-sum(repmat(bpol,1,sum(ind(t,:))).*regPOLt,1)',0,sigmak(ind(t,:)));
    else
        PSI=normcdf(boldM(ind(t,:))-boldG(ind(t,:))-timeeffect(t)-fixedeffect(ind(t,:)),0,sigmak(ind(t,:)));
    end    
    x=X(t,ind(t,:));
    aux=find(ind(t,:)==1);

    
    % gradient for b0
    dbb0=Pold\invV;
    Ddbb0=dbb0(N+1:end,:)'-dbb0(1:N,:)';

      
    xPSI=zeros(length(x),1);
    xPSI(find(x==1))=1./PSI(find(x==1));
    xPSI(find(x==0))=-1./(1-PSI(find(x==0)));
    % xPSI=[x'./PSI-(1-x')./(1-PSI)];
    
    if kpol>=1
        auxF=[normpdf((boldM(ind(t,:))-boldG(ind(t,:))-timeeffect(t)-fixedeffect(ind(t,:))-sum(repmat(bpol,1,sum(ind(t,:))).*regPOLt,1)')./sigmak(ind(t,:))).*xPSI./sigmak(ind(t,:))]';
        gradsigmakt = - normpdf([boldM(ind(t,:))-boldG(ind(t,:))-timeeffect(t)-fixedeffect(ind(t,:))-sum(repmat(bpol,1,sum(ind(t,:))).*regPOLt,1)']./sigmak(ind(t,:))) .* [boldM(ind(t,:))-boldG(ind(t,:))-timeeffect(t)-fixedeffect(ind(t,:))-sum(repmat(bpol,1,sum(ind(t,:))).*regPOLt,1)'] ./ (sigmak(ind(t,:)).^2);
    else
        auxF=[normpdf((boldM(ind(t,:))-boldG(ind(t,:))-timeeffect(t)-fixedeffect(ind(t,:)))./sigmak(ind(t,:))).*xPSI./sigmak(ind(t,:))]';
        gradsigmakt = - normpdf([boldM(ind(t,:))-boldG(ind(t,:))-timeeffect(t)-fixedeffect(ind(t,:))]./sigmak(ind(t,:))) .* [boldM(ind(t,:))-boldG(ind(t,:))-timeeffect(t)-fixedeffect(ind(t,:))] ./ (sigmak(ind(t,:)).^2);
    end
    GRADb0=GRADb0+sum(repmat(auxF,2*N,1).*Ddbb0(:,ind(t,:)),2);

    
    % gradient for sigmak
    GRADsigmak(ind(t,:)) = GRADsigmak(ind(t,:)) + gradsigmakt .* xPSI;
    

    % gradient for gamma (correlation of prior VCM)
    for j=1:k
        dbbdist=diag(S.*diagsqrtVCM)*((v-eye(N)).*squeeze(distance(:,:,j)))*diag(diagsqrtVCM.*S)*invV(1:N,1:N);
        dbbdist=dbb0*[dbbdist*(b0(1:N)-bold(1:N));dbbdist*(b0(N+1:end)-bold(N+1:end))];
        dbbdist=dbbdist(N+1:end)-dbbdist(1:N);
        GRADBdist(j)=GRADBdist(j)+sum(auxF.*dbbdist(ind(t,:))');
    end
    
    
    %gradient for bpol
    if kpol>=1
        GRADBpol = GRADBpol - sum(repmat(auxF',1,kpol).*regPOLt',1)';
    end
        
    
    % gradient for S
    dbS=zeros(2*N,N);
    invVb=[invV(1:N,1:N)*(bold(1:N)-b0(1:N));invV(1:N,1:N)*(bold(N+1:end)-b0(N+1:end))];
    for j=1:N
        dVS=sparse(N,N); dVS(:,j)=VV(:,j).*S; dVS(j,:)=VV(j,:).*S'; dVS(j,j)=2*dVS(j,j);
        dbS(:,j)=[smvp(dVS,invVb(1:N));smvp(dVS,invVb(N+1:end))];
        % dbS(:,j)=[dVS*invVb(1:N);dVS*invVb(N+1:end)];
    end
    dbS=dbb0*dbS;
    dbS=dbS(N+1:end,:)-dbS(1:N,:);
    GRADS=GRADS+sum(repmat(auxF,N,1).*dbS(ind(t,:),:)',2);
    
    
    % gradient for FE and TE
    if FT==1 
        GRADFE(ind(t,:)) = GRADFE(ind(t,:)) - auxF';
    elseif FT==2
        GRADFE(ind(t,:)) = GRADFE(ind(t,:)) - auxF';
        GRADTE(t)=sum(-auxF);
    end

    
    bold=bnew;
    Pold=Pnew;
end


clear bold bnew Pold Pnew

% priors and gradients of priors
GRADb0=GRADb0-(b0 - [repmat(b0mean(1),N,1);repmat(b0mean(2),N,1)])/(b0std^2);
GRADS=GRADS + scaleS.*(S.^(-3))-(degS+1)./S;
GRADsigmak=GRADsigmak + scalek.*(sigmak.^(-3))-(degk+1)./sigmak;

% gradient
if FT==0;
    g=[-GRADb0;-GRADS.*exp(estpr(2*N+1:3*N));-GRADsigmak.*exp(estpr(3*N+1:4*N));-GRADBdist.*exp(estpr(4*N+1:4*N+k));-GRADBpol];
elseif FT==1
    GRADFE=GRADFE-(fixedeffect - FEmean)./(FEstd^2);
    g=[-GRADb0;-GRADS.*exp(estpr(2*N+1:3*N));-GRADsigmak.*exp(estpr(3*N+1:4*N));-GRADBdist.*exp(estpr(4*N+1:4*N+k));-GRADBpol;-GRADFE];
elseif FT==2;
    GRADFE=GRADFE-(fixedeffect - FEmean)./(FEstd^2);
    g=[-GRADb0;-GRADS.*exp(estpr(2*N+1:3*N));-GRADsigmak.*exp(estpr(3*N+1:4*N));-GRADBdist.*exp(estpr(4*N+1:4*N+k));-GRADBpol;-GRADFE;-GRADTE];
end


badg=0;
if any(abs(g)>1e15)==1
    disp('bad gradient ------------------------')
    g(find(abs(g)>1e15))=0;
    badg=1;
end
%profile viewer