clear all

download_data

% dimensions and missing values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[T,N]=size(open);
ind_open=open~=-999;                 % indicator matrix for openess data (=0 if missing data, =1 otherwise)
ind_growth=growth~=-999;             % indicator matrix for growth  data (=0 if missing data, =1 otherwise)
ind=logical(ind_open.*ind_growth);   % indicator matrix (=1 if data for both openess and growth are not missing, =0 otherwise)

TT=T-1;                              % get rid of the first obs because there are no data for growth and creates a collinearity problem
growth2=growth(2:end,:);
open2=open(2:end,:);
ind2=logical(ind(2:end,:));


% determinants of the political cost
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
polity=polity(2:end,:);
PCGDP=PCGDP(1:end-1,:);
CivilWars=CivilWars(2:end,:);
SovietControl=SovietControl(2:end,:);
PolityNeigh=PolityNeigh(2:end,:);
CivilWarsNeigh=CivilWarsNeigh(2:end,:);
INF=INF(1:end-1,:);
DEF=DEF(1:end-1,:);
EDUC=education(2:end,:);

ind_polity=polity~=-999;                 
ind_PCGDP=PCGDP~=-999;  
ind_CivilWars=CivilWars~=-999;
ind_SovietControl=SovietControl~=-999;
ind_PolityNeigh=PolityNeigh~=-999;                 
ind_CivilWarsNeigh=CivilWarsNeigh~=-999;
ind_INF=INF~=-999;
ind_DEF=DEF~=-999/100;
ind_EDUC=EDUC~=-999;

ind2=logical(ind(2:end,:).*ind_polity.*ind_PCGDP.*ind_CivilWars);

% demeaning variables
politybar=mean(polity(ind2));
polity=polity-politybar;

PCGDPbar=mean(PCGDP(ind2));
PCGDP=PCGDP-PCGDPbar;

CivilWarsbar=mean(CivilWars(ind2));
CivilWars=CivilWars-CivilWarsbar;

SovietControlbar=mean(SovietControl(ind2));
SovietControl=SovietControl-SovietControlbar;

PolityNeighbar=mean(PolityNeigh(ind2));
PolityNeigh=PolityNeigh-PolityNeighbar;

CivilWarsNeighbar=mean(CivilWarsNeigh(ind2));
CivilWarsNeigh=CivilWarsNeigh-CivilWarsNeighbar;

INFbar=mean(INF(ind2));
INF=INF-INFbar;

DEFbar=mean(DEF(ind2));
DEF=DEF-DEFbar;

EDUCbar=mean(EDUC(ind2));
EDUC=EDUC-EDUCbar;


regPOL=[polity;PCGDP;CivilWars];
kpol=3;         % # number of determinants of the political cost
flagGDP=2;      % position of PCGDP
FT=1;           % Fixed and time effect (1 = fixed effect, 2 = fixed and time effect)

% determinants of the prior spatial correlation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
k=1;
distance=zeros(N,N,k);
distance(:,:,1)=dist/1000;


% coefficients parameterizing the priors of the econometrician
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
b0mean_G=0.02+.0075;    % econometrician prior mean on agents' initial beliefs   
b0mean_M=0.02-.0075;    % econometrician prior mean on agents' initial beliefs
b0std=.02;              % econometrician prior std on agents' initial beliefs
Smean=sqrt(.0696);      % econometrician prior mean on the std of agents' initial beliefs
Sstd=sqrt(.0696);       % econometrician prior std on the std of agents' initial beliefs
sigmakmean=.01;         % econometrician mean of std of cost of staying open
sigmakstd=.01;          % econometrician std of std of cost of staying open
FEmean = 0;             % econometrician prior mean of fixed effect in political cost
FEstd = 0.02;           % econometrician prior std of fixed effect in political cost

[scalek,degk]=inverse_gamma_specification(sigmakmean,sigmakstd,1);
[scaleS,degS]=inverse_gamma_specification(Smean,Sstd,1);
minsigmak=.0001;
minS=0.001;
maxcorr=.99;
priorvec=[b0mean_G b0mean_M b0std scaleS degS minS degk scalek minsigmak maxcorr FEmean FEstd]';


% a few object that are used in the minimization and it is convenient to
% define in advance
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
XX=zeros(2*N,2*N*TT);
XY=zeros(2*N,TT);
for t=1:TT
    
    opent=sparse([diag(1-open2(t,:)) diag(open2(t,:))]); opent(logical(1-ind2(t,:)),:)=0;
    growtht=growth2(t,:)'; growtht(logical(1-ind2(t,:)))=0;
    
    invVCM=zeros(N);
    invVCM(ind2(t,:),ind2(t,:))=eye(sum(ind2(t,:)))/VCM(ind2(t,:),ind2(t,:));
    
    XXt=opent'*invVCM*opent;
    XX(:,(t-1)*2*N+1:t*2*N)=XXt;
    
    XYt=opent'*invVCM*growtht;
    XY(:,t)=XYt;
   
end
XX=sparse(XX);


% starting values for minimization 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% option 1
% for the baseline model, we use the following starting values
% cd data
% load StartingValuesBaseline
% cd ..
% x0=StartingValuesBaseline;
% [fh,xh,gh,Hessian,itct,fcount,retcodeh] = csminwel('logpost_multicountry_joint',x0,Hessian0,'gradient',1e-6,3000,growth2,open2,TT,N,ind2,priorvec,distance,k,kpol,sqrt(diag(VCM)),XX,XY,regPOL,FT);

% option 2
% starting values are selected at random and then ranked based on their
% posterior values (this option is computationally much more expensive, much 
% more robust and should be used for any model other than the baseline
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:133
    STD(i)=std(growth2(ind2(:,i),i));
end
STD(find(STD==0))=.03;
STD(isnan(STD))=.03;

np=4*N+k+kpol+N*(FT==1)+(N+TT)*(FT==2);
const0=.05;
const1=.5;
Hessian0=const0*eye(np); Hessian0(4*N+1:4*N+k,4*N+1:4*N+k)=const1*eye(k);

M=5000;
x0=zeros(np,M);
logpostDRRAWS=zeros(M,1);
for i=1:M
    divsig=rand(N,1).*3+7;
    divS=rand(N,1)+2;
    bdist0=abs(mvnrnd([.5],diag([.25].^2)));
    if kpol>=1
        bpol0=mvnrnd(zeros(1,kpol),(.001^2)*eye(kpol));
    else
        bpol0=[];
    end
    if FT==0
        x0(:,i)=[normrnd(b0mean_G-.005,b0std/2,[1 N]) normrnd(b0mean_M-.0075,b0std/2,[1 N]) log(sqrt(0.0696./divS')-minS) log(STD./divsig'-minsigmak)... 
            log(bdist0) bpol0]';
    elseif FT==1;
        x0(:,i)=[normrnd(b0mean_G-.005,b0std/2,[1 N]) normrnd(b0mean_M-.0075,b0std/2,[1 N]) log(sqrt(0.0696./divS')-minS) log(STD./divsig'-minsigmak)...
            log(bdist0) bpol0 normrnd(0,FEstd/2,[1 N])]';
    elseif FT==2;
        x0(:,i)=[normrnd(b0mean_G-.005,b0std/2,[1 N]) normrnd(b0mean_M-.0075,b0std/2,[1 N]) log(sqrt(0.0696./divS')-minS) log(STD./divsig'-minsigmak)...
            log(bdist0) bpol0 normrnd(0,FEstd/2,[1 N]) normrnd(0,.005,[1 TT])]';
    end
    logpostDRAWS(i)=logpost_multicountry_joint(x0(:,i),growth2,open2,TT,N,ind2,priorvec,distance,k,kpol,sqrt(diag(VCM)),XX,XY,regPOL,FT);
end

[values,order]=sort(logpostDRAWS);

% 
J=25;       % # of minimizations (launched from different starting values)
x0=x0(:,order(1:J));
clear logpostDRAWS
xh=zeros(np,J);
fh=zeros(J,1);
itct=zeros(J,1);
for i=1:J
    [fh(i),xh(:,i),gh,Hessian,itct(i),fcount,retcodeh] = csminwel('logpost_multicountry_joint',x0(:,i),Hessian0,'gradient',1e-6,3000,growth2,open2,TT,N,ind2,priorvec,distance,k,kpol,sqrt(diag(VCM)),XX,XY,regPOL,FT);
    if i==1
        HessianMAX=Hessian;
    else
        if fh(i)<min(fh(fh~=0))
            HessianMAX=Hessian;
        end
    end
end